@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # coding: utf-8
 
-# In[428]:
+# In[4]:
 
 
 try:
@@ -12,10 +12,10 @@ except:
         pass
 
 
-# In[429]:
+# In[5]:
 
 
-version = '0.75'
+version = '0.78'
 
 import re
 import tempfile
@@ -33,7 +33,7 @@ import time
 
 # # Helper Functions
 
-# In[430]:
+# In[6]:
 
 
 def is_testing():
@@ -43,7 +43,7 @@ def is_testing():
         return False
 
 
-# In[431]:
+# In[7]:
 
 
 def isnotebook():
@@ -54,7 +54,7 @@ def isnotebook():
     return False
 
 
-# In[432]:
+# In[8]:
 
 
 def testPrint(*args):
@@ -64,13 +64,13 @@ def testPrint(*args):
 
 # # Timing
 
-# In[433]:
+# In[9]:
 
 
 timeSinceLastCallDict = {}
 
 
-# In[434]:
+# In[10]:
 
 
 def timeSinceLastCall(name):
@@ -83,7 +83,7 @@ def timeSinceLastCall(name):
     return name + ": " + time_s
 
 
-# In[435]:
+# In[11]:
 
 
 if is_testing():
@@ -92,7 +92,7 @@ if is_testing():
 
 # # Requiremtns
 
-# In[436]:
+# In[12]:
 
 
 requirements = []
@@ -124,7 +124,7 @@ import yaml
 
 # # Strings
 
-# In[437]:
+# In[13]:
 
 
 def sFloat0(f): return "{:0.0f}".format(f)
@@ -154,7 +154,7 @@ if is_testing():
     print(sFloatSci5(1.234567))
 
 
-# In[438]:
+# In[14]:
 
 
 def strUuid4():
@@ -165,14 +165,14 @@ def strUuid4():
     return uuid
 
 
-# In[439]:
+# In[15]:
 
 
 if is_testing():
     print(strUuid4())
 
 
-# In[440]:
+# In[16]:
 
 
 def strRand(n, num=True, upperCase=True, lowerCase=True):
@@ -184,7 +184,7 @@ def strRand(n, num=True, upperCase=True, lowerCase=True):
     return ''.join(random.choice(pool) for _ in range(n))
 
 
-# In[441]:
+# In[17]:
 
 
 if is_testing():
@@ -193,7 +193,7 @@ if is_testing():
     print(strRand(20, upperCase=False, lowerCase=False))
 
 
-# In[442]:
+# In[18]:
 
 
 def strMd5Pickle4(obj):
@@ -203,14 +203,14 @@ def strMd5Pickle4(obj):
     return md5.hexdigest()
 
 
-# In[443]:
+# In[19]:
 
 
 if is_testing():
     print(strMd5Pickle4(['a', 'b']))
 
 
-# In[444]:
+# In[20]:
 
 
 def strMd5String(s):
@@ -219,7 +219,7 @@ def strMd5String(s):
     return md5.hexdigest()
 
 
-# In[445]:
+# In[21]:
 
 
 if is_testing():
@@ -228,7 +228,7 @@ if is_testing():
 
 # # Multiprocess / Thread
 
-# In[446]:
+# In[22]:
 
 
 def mapThread(func, args, n=8):
@@ -253,14 +253,14 @@ if is_testing():
 
 # # Lists
 
-# In[447]:
+# In[23]:
 
 
 def liChunks(l, chunk_size):
     return [l[i:i + chunk_size] for i in range(0, len(l), chunk_size)]
 
 
-# In[448]:
+# In[24]:
 
 
 if is_testing():
@@ -269,7 +269,7 @@ if is_testing():
     print(liChunks(['a', 'b', 'c', 'd', 'e', 'f'], 3))
 
 
-# In[449]:
+# In[25]:
 
 
 def liSplit(a, number_of_splits):
@@ -277,7 +277,7 @@ def liSplit(a, number_of_splits):
     return [a[start:end] for start, end in zip(idxs[:-1], idxs[1:])]
 
 
-# In[450]:
+# In[26]:
 
 
 if is_testing():
@@ -289,7 +289,7 @@ if is_testing():
     print(liSplit(['a', 'b', 'c', 'd', 'e'], 6))
 
 
-# In[451]:
+# In[27]:
 
 
 def liJoin(l):
@@ -299,14 +299,14 @@ def liJoin(l):
     return out
 
 
-# In[452]:
+# In[28]:
 
 
 if is_testing():
     print(liJoin([['a', 'b', 'c'], ['d', 'e', 'f']]))
 
 
-# In[453]:
+# In[29]:
 
 
 def liFlatten(l):
@@ -323,7 +323,7 @@ def liFlatten(l):
         return l
 
 
-# In[454]:
+# In[30]:
 
 
 if is_testing():
@@ -333,7 +333,7 @@ if is_testing():
 
 # # List of Strings
 
-# In[455]:
+# In[31]:
 
 
 regex_cache = {}
@@ -343,35 +343,35 @@ def sListFilter(l, re_include=".*", re_exclude='^\b$'):
     return [e for e in l if regex_cache[re_include].search(e) and not regex_cache[re_exclude].search(e)]
 
 
-# In[456]:
+# In[32]:
 
 
 if is_testing():
     sListFilter(['good x', 'other good x', 'without the letter before y'])
 
 
-# In[457]:
+# In[33]:
 
 
 if is_testing():
     print(sListFilter(['good x', 'other good x', 'without the letter before y'], re_include='x'))
 
 
-# In[458]:
+# In[34]:
 
 
 if is_testing():
     print(sListFilter(['not enough x', 'enough xx', 'enough xx and also z'], re_include='xx+'))
 
 
-# In[459]:
+# In[35]:
 
 
 if is_testing():
     print(sListFilter(['not enough x', 'enough xx', 'enough xx and also z'], re_include='xx+', re_exclude='z'))
 
 
-# In[460]:
+# In[36]:
 
 
 def lCsv(l, force_oneline=False):
@@ -387,21 +387,21 @@ def lCsv(l, force_oneline=False):
     return ", ".join([str(e) for e in l])
 
 
-# In[461]:
+# In[37]:
 
 
 if is_testing():
     print(lCsv(['a', 'b', 1, 3.14]))
 
 
-# In[462]:
+# In[38]:
 
 
 if is_testing():
     print(lCsv([['a', 'b'], 1, 3.14]))
 
 
-# In[463]:
+# In[39]:
 
 
 if is_testing():
@@ -410,7 +410,7 @@ if is_testing():
 
 # # Text files
 
-# In[464]:
+# In[40]:
 
 
 def txtread(path):
@@ -431,7 +431,7 @@ def txtwrite(path, txt):
             return f.write(txt)
 
 
-# In[465]:
+# In[41]:
 
 
 if is_testing():
@@ -440,21 +440,21 @@ if is_testing():
 
 # # YAML files
 
-# In[466]:
+# In[42]:
 
 
 def yamlwrite(path, obj):
     txtwrite(path, yaml.dump(obj))
 
 
-# In[467]:
+# In[43]:
 
 
 def yamlread(path):
     return yaml.safe_load(txtread(path=path))
 
 
-# In[468]:
+# In[44]:
 
 
 if is_testing():
@@ -467,7 +467,7 @@ if is_testing():
 
 # # List of Strings
 
-# In[469]:
+# In[45]:
 
 
 regex_cache = {}
@@ -479,35 +479,35 @@ def sListFilter(listOfStrings, re_include=".*", re_exclude='^\b$'):
 
 # ### Examples
 
-# In[470]:
+# In[46]:
 
 
 if is_testing():
     print(sListFilter(['good x', 'other good x', 'without the letter before y']))
 
 
-# In[471]:
+# In[47]:
 
 
 if is_testing():
     print(sListFilter(['good x', 'other good x', 'without the letter before y'], re_include='x'))
 
 
-# In[472]:
+# In[48]:
 
 
 if is_testing():
     print(sListFilter(['not enough x', 'enough xx', 'enough xx and also z'], re_include='xx+'))
 
 
-# In[473]:
+# In[49]:
 
 
 if is_testing():
     print(sListFilter(['not enough x', 'enough xx', 'enough xx and also z'], re_include='xx+', re_exclude='z'))
 
 
-# In[474]:
+# In[50]:
 
 
 if is_testing():
@@ -516,7 +516,7 @@ if is_testing():
 
 # # GCP Google Cloud Platform
 
-# In[475]:
+# In[51]:
 
 
 import warnings
@@ -540,7 +540,7 @@ def _gsBlob(path):
     return blob
 
 
-# In[476]:
+# In[52]:
 
 
 def gsGsToHttp(path):
@@ -550,7 +550,7 @@ def gsHttpToGs(path):
     return path.replace("https://storage.cloud.google.com/", "gs://").split("?")[0] # "?" is for possible "?authuser=0"
 
 
-# In[477]:
+# In[53]:
 
 
 def gsLs(path):
@@ -560,7 +560,7 @@ def gsLs(path):
     return ['gs://' + bucket + '/' + blob.name for blob in client.list_blobs(prefix=prefix)]
 
 
-# In[478]:
+# In[54]:
 
 
 def gsExists(path):
@@ -570,7 +570,7 @@ def gsExists(path):
     return client.blob(prefix).exists()
 
 
-# In[479]:
+# In[55]:
 
 
 def gsUploadFile(gsPath, path):
@@ -578,7 +578,7 @@ def gsUploadFile(gsPath, path):
     blob.upload_from_filename(path)
 
 
-# In[480]:
+# In[56]:
 
 
 def gsTxtread(path):
@@ -590,7 +590,7 @@ def gsTxtread(path):
     return blob.download_as_string().decode()
 
 
-# In[481]:
+# In[57]:
 
 
 def gsTxtwrite(path, s):
@@ -598,7 +598,7 @@ def gsTxtwrite(path, s):
     return blob.upload_from_string(s)
 
 
-# In[482]:
+# In[58]:
 
 
 def gsImwrite(path, img):
@@ -607,7 +607,7 @@ def gsImwrite(path, img):
     blob.upload_from_string(encimg.tobytes())
 
 
-# In[483]:
+# In[59]:
 
 
 def gsImread(path):
@@ -616,7 +616,7 @@ def gsImread(path):
     return cv2.imdecode(raw, 1)[:, :, [2, 1, 0]]
 
 
-# In[484]:
+# In[60]:
 
 
 def gsPklWrite(path, obj):
@@ -624,7 +624,7 @@ def gsPklWrite(path, obj):
     return blob.upload_from_string(pickle.dumps(obj, protocol=4))
 
 
-# In[485]:
+# In[61]:
 
 
 def gsPklRead(path):
@@ -634,7 +634,7 @@ def gsPklRead(path):
 
 # # Pickle
 
-# In[486]:
+# In[62]:
 
 
 def pklwrite(path, obj):
@@ -647,7 +647,7 @@ def pklwrite(path, obj):
             return pickle.dump(obj, f, protocol=4)
 
 
-# In[487]:
+# In[63]:
 
 
 def pklread(path):
@@ -658,7 +658,7 @@ def pklread(path):
             return pickle.load(f)
 
 
-# In[488]:
+# In[64]:
 
 
 if is_testing():
@@ -671,7 +671,7 @@ if is_testing():
 
 # # Images
 
-# In[489]:
+# In[65]:
 
 
 def imread(path):
@@ -705,7 +705,7 @@ def impad(img, top=0, bottom=0, left=0, right=0, color=255):
     return np.pad(img, [(top, bottom), (left, right), (0, 0)], 'constant', constant_values=color)
 
 
-# In[490]:
+# In[66]:
 
 
 def imshow(array, scale=1):
@@ -720,7 +720,7 @@ def imshow(array, scale=1):
         cv2.waitKey()
 
 
-# In[491]:
+# In[67]:
 
 
 if is_testing():
@@ -728,14 +728,14 @@ if is_testing():
     imshow(img)
 
 
-# In[492]:
+# In[ ]:
 
 
 if is_testing():
     imshow('http://via.placeholder.com/70.png')
 
 
-# In[493]:
+# In[68]:
 
 
 def imscaleNN(img, s):
@@ -745,7 +745,7 @@ def imscaleBic(img, s):
     return cv2.resize(img, None, fx=s, fy=s, interpolation=cv2.INTER_CUBIC)
 
 
-# In[494]:
+# In[69]:
 
 
 if is_testing():
@@ -764,7 +764,7 @@ if is_testing():
         imshow(out_large_interbic)
 
 
-# In[495]:
+# In[70]:
 
 
 def imRepeat(img, height, width):
@@ -778,7 +778,7 @@ def imRepeat(img, height, width):
     return img
 
 
-# In[496]:
+# In[71]:
 
 
 if is_testing():
@@ -786,33 +786,33 @@ if is_testing():
     imshow(imRepeat(img_random_color_3x5, 7, 11), scale=10)
 
 
-# In[497]:
+# In[72]:
 
 
 def imNewWhite(height, width):
     return np.ones((height, width, 3)).astype(np.uint8) * 255
 
 
-# In[498]:
+# In[73]:
 
 
 if is_testing(): imshow(imNewWhite(10, 500) - 30)
 
 
-# In[499]:
+# In[74]:
 
 
 def imNewBlack(height, width):
     return np.zeros((height, width, 3)).astype(np.uint8)
 
 
-# In[500]:
+# In[75]:
 
 
 if is_testing(): imshow(imNewBlack(10, 500))
 
 
-# In[501]:
+# In[76]:
 
 
 def imAddNoiseGauss(img, std):
@@ -824,7 +824,7 @@ def imAddNoiseGauss(img, std):
 if is_testing(): imshow(imAddNoiseGauss(imread('http://via.placeholder.com/150.png'), 10))
 
 
-# In[502]:
+# In[ ]:
 
 
 def imJpgDegradation(img, quality):
@@ -837,7 +837,7 @@ def imJpgDegradation(img, quality):
 if is_testing(): imshow(imJpgDegradation(imread('http://via.placeholder.com/150.png'), 15))
 
 
-# In[503]:
+# In[ ]:
 
 
 def imCropCenter(img, size):
@@ -852,58 +852,68 @@ def imCropCenter(img, size):
     return img[h_start:h_end, w_start:w_end]
 
 
-# In[504]:
+# In[ ]:
 
 
 if is_testing(): 
     imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 10))
 
 
-# In[505]:
+# In[ ]:
 
 
 if is_testing(): 
     imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 49))
 
 
-# In[506]:
+# In[ ]:
 
 
 if is_testing(): 
     imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 50))
 
 
-# In[507]:
+# In[ ]:
 
 
 if is_testing(): 
     imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 100))
 
 
-# In[508]:
+# In[77]:
 
 
-def imCropRandom(img, size):
+def imCropRandom(img, size, auto_size=True):
     h, w, c = img.shape
 
-    h_start = np.random.randint(0, h-size)
-    h_end = h_start + size
+    if auto_size and h <= size:
+        h_start = 0
+        h_end = h
+    else:
+        h_start = np.random.randint(0, h-size)
+        h_end = h_start + size
 
-    w_start = np.random.randint(0, w-size)
-    w_end = w_start + size
+    if auto_size and w <= size:
+        w_start = 0
+        w_end = w
+    else:
+        w_start = np.random.randint(0, w-size)
+        w_end = w_start + size
 
     return img[h_start:h_end, w_start:w_end]
 
 
-# In[509]:
+# In[ ]:
 
 
 if is_testing(): 
     imshow(imCropRandom(imread('http://via.placeholder.com/40.png'), 10))
     imshow(imCropRandom(imread('http://via.placeholder.com/40.png'), 39))
+    imshow(imCropRandom(imread('http://via.placeholder.com/40.png'), 40))
+    imshow(imCropRandom(imread('http://via.placeholder.com/40.png'), 41))
 
 
-# In[510]:
+# In[78]:
 
 
 def imGallery(imgs, pad=0):
@@ -934,7 +944,7 @@ def imGallery(imgs, pad=0):
     return out
 
 
-# In[511]:
+# In[79]:
 
 
 if is_testing():
@@ -942,49 +952,49 @@ if is_testing():
     listOfDummyImagesBlackToWhite = [imNewBlack(10, 10) + 25 * i for i in range(10)]
 
 
-# In[512]:
+# In[80]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:1]))
 
 
-# In[513]:
+# In[81]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:2]))
 
 
-# In[514]:
+# In[82]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:2], pad=1))
 
 
-# In[515]:
+# In[83]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:5], pad=1))
 
 
-# In[516]:
+# In[84]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:9], pad=1))
 
 
-# In[517]:
+# In[85]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:10], pad=1))
 
 
-# In[518]:
+# In[86]:
 
 
 if is_testing():
@@ -992,16 +1002,80 @@ if is_testing():
     imshow(imGallery([img40x20, img40x20, img40x20], pad=1))
 
 
+# In[87]:
+
+
+def imAddTextHeader(img, text, h=None, size=None):
+    from PIL import Image, ImageDraw, ImageFont
+    
+    if h is None:
+        h = int(np.ceil(img.shape[0] * 0.1))
+    if size is None:
+        size = h
+    
+    text_img = np.ones((h, img.shape[1], img.shape[2]), dtype=img.dtype) * 255
+    im = Image.fromarray(text_img)
+
+    draw = ImageDraw.Draw(im)
+    
+    w = text_img.shape[1]
+    h = text_img.shape[0]
+    centered=True
+    middle = True
+    text_width, text_height = draw.textsize(text)
+    draw.text((w/2 - (text_width/2) * centered, text_height/2 - h/2)
+        ,text, fill="black")
+    
+    text = np.array(im)[:, :, :3]
+    
+    img = np.concatenate([text, img], axis=0)
+    
+    return img
+
+if is_testing():
+    imshow(imAddTextHeader(imNewBlack(100, 100), 'black image'))
+    imshow(imAddTextHeader(imNewBlack(100, 100), 'black image', h = 20))
+
+
+# # pytorch
+
+# In[88]:
+
+
+def ptT(img_np_uint8):
+    import torch
+    img = img_np_uint8
+    assert img.dtype == np.uint8, img.dtype
+    return torch.Tensor(np.expand_dims(img.transpose([2, 0, 1]), 0) / 255.0).to(torch.float32)
+
+def ptUint8(img_tensor):
+    import torch
+    img = img_tensor
+    assert img.dtype == torch.float32, img.dtype
+    return np.clip(img.detach().cpu().numpy()[0].transpose([1, 2, 0]) * 255, 0, 255).astype(np.uint8)
+    
+
+
+# In[89]:
+
+
+if is_testing():
+    iOrig = imNewWhite(10, 10)
+    t = ptT(iOrig)
+    i = ptUint8(t)
+    print('iOrig', np.min(iOrig), np.max(iOrig), iOrig.shape, 't', t.min(), t.max(), t.shape, 'i', i.min(), i.max(), i.shape, 'err', np.mean(np.abs(iOrig - i)))
+
+
 # # Files
 
-# In[519]:
+# In[90]:
 
 
 def fiFindByWildcard(wildcard):
     return natsort.natsorted(glob.glob(wildcard, recursive=True))
 
 
-# In[520]:
+# In[91]:
 
 
 if is_testing():
@@ -1030,7 +1104,7 @@ if is_testing():
         imshow(imGallery(imgs))
 
 
-# In[521]:
+# In[92]:
 
 
 def fiRemove(path):
@@ -1058,7 +1132,7 @@ def fiAgeHours(path):
     return (time.time() - os.stat(path)[stat.ST_MTIME]) / 3600
 
 
-# In[522]:
+# In[93]:
 
 
 if is_testing():
@@ -1090,7 +1164,7 @@ if is_testing():
         print("files: " + "\n".join(fiFindByWildcard(os.path.join(d, "**", "*"))))
 
 
-# In[523]:
+# In[94]:
 
 
 def fiUnzip(zip_path, out_dir=None, remove=False):
@@ -1105,7 +1179,7 @@ def fiUnzip(zip_path, out_dir=None, remove=False):
         zip.extractall(out_dir)
 
 
-# In[524]:
+# In[95]:
 
 
 def fiJoin(*args):
@@ -1117,7 +1191,7 @@ if is_testing():
 
 # # Jupyter
 
-# In[525]:
+# In[96]:
 
 
 def jnSettings():
@@ -1128,7 +1202,7 @@ if is_testing():
     jnSettings()
 
 
-# In[526]:
+# In[97]:
 
 
 def jnNbFullWidth():
@@ -1142,7 +1216,7 @@ def jnPandasSettings():
     pd.set_option('display.max_colwidth', 100000)
 
 
-# In[527]:
+# In[98]:
 
 
 def jnImageRenderingPixelated():
@@ -1154,7 +1228,7 @@ def jnImageRenderingAuto():
     display(HTML('<style type = text/css> img { image-rendering: auto; } </style>'))
 
 
-# In[528]:
+# In[99]:
 
 
 def jnDefault():
@@ -1172,7 +1246,7 @@ if is_testing():
 
 
 
-# In[529]:
+# In[100]:
 
 
 def jnHtmlRow(strList, width=100):
@@ -1185,7 +1259,7 @@ if is_testing():
     
 
 
-# In[530]:
+# In[101]:
 
 
 def jnImgToHtmlB64(img):
@@ -1220,7 +1294,7 @@ def jnImageRow(imgs, names=None, width=100, external=False, title_img=None, titl
     display(HTML(html))
 
 
-# In[531]:
+# In[ ]:
 
 
 if is_testing():
@@ -1228,7 +1302,7 @@ if is_testing():
    
 
 
-# In[532]:
+# In[ ]:
 
 
 if is_testing():
@@ -1238,7 +1312,7 @@ if is_testing():
     jnImageRow(imgs, external=True, title='test')
 
 
-# In[533]:
+# In[ ]:
 
 
 
@@ -1248,7 +1322,7 @@ if is_testing():
 
 # # Shell
 
-# In[534]:
+# In[ ]:
 
 
 def shell(cmd):
@@ -1263,7 +1337,7 @@ def shell(cmd):
 
 # # GIT
 
-# In[535]:
+# In[ ]:
 
 
 def gitGetHash():
@@ -1273,7 +1347,7 @@ if is_testing():
     print(gitGetHash())
 
 
-# In[536]:
+# In[ ]:
 
 
 def gitGetBranch():
@@ -1283,7 +1357,7 @@ if is_testing():
     print(gitGetBranch())
 
 
-# In[537]:
+# In[ ]:
 
 
 def gitGetMessage():
@@ -1293,7 +1367,7 @@ if is_testing():
     print(gitGetMessage())
 
 
-# In[538]:
+# In[ ]:
 
 
 def gitGetAllFiles():
@@ -1303,7 +1377,7 @@ if is_testing():
     print(gitGetAllFiles())
 
 
-# In[539]:
+# In[ ]:
 
 
 def gitListByTime(path=None):
@@ -1319,18 +1393,18 @@ if is_testing():
     print(gitListByTime()[-3:])
 
 
-# In[540]:
+# In[ ]:
 
 
 def gitTime(path):
-    _time = lambda x: shell('git log -1 --format="%ad" -- {filename}'.format(filename=x)).replace("\n", "")
+    _time = lambda x: shell('cd $(dirname {filename}) && git log -1 --format="%ad" -- {filename}'.format(filename=x)).replace("\n", "")
     return _time(path)
     
 if is_testing():
     print(gitTime("__init__.py"))
 
 
-# In[541]:
+# In[ ]:
 
 
 def gitTimeRaw(path):
@@ -1341,7 +1415,7 @@ if is_testing():
     print(gitTimeRaw("__init__.py"))
 
 
-# In[542]:
+# In[ ]:
 
 
 def shellSortDirsBySize():
@@ -1353,14 +1427,14 @@ if is_testing():
 
 # # Package Files
 
-# In[543]:
+# In[ ]:
 
 
 if is_testing():
     package_files = {}
 
 
-# In[544]:
+# In[ ]:
 
 
 def shell(cmd):
@@ -1368,7 +1442,7 @@ def shell(cmd):
     return result.stdout.decode()
 
 
-# In[545]:
+# In[ ]:
 
 
 if is_testing():
@@ -1400,7 +1474,7 @@ setup(
 )""".format(requirements=", ".join(["'" + r + "'" for r in requirements]), version=version)
 
 
-# In[546]:
+# In[ ]:
 
 
 if is_testing():
@@ -1410,7 +1484,7 @@ description-file = README.md
 """
 
 
-# In[547]:
+# In[ ]:
 
 
 if is_testing():
@@ -1433,14 +1507,14 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE."""
 
 
-# In[548]:
+# In[ ]:
 
 
 if is_testing():
     package_files['README.md'] = ""
 
 
-# In[549]:
+# In[ ]:
 
 
 if is_testing():
@@ -1449,7 +1523,7 @@ if is_testing():
     
 
 
-# In[550]:
+# In[ ]:
 
 
 if is_testing():
@@ -1460,7 +1534,7 @@ lx/__init__.py
 """
 
 
-# In[551]:
+# In[ ]:
 
 
 if is_testing():
@@ -1483,3 +1557,21 @@ if is_testing():
 
 
 
+
+# In[ ]:
+
+
+
+
+
+# In[ ]:
+
+
+
+
+
+# In[ ]:
+
+
+
+
